(*
 * Copyright 2005 Toshiyuki Maeda
 *
 * This file is part of TALK.
 *
 * TALK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * TALK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)

{
open Asmparser
}

let ws = [' ' '\t']
let nl = ['\n' '\r']
let dig = ['0' - '9']
let lwr = ['a' - 'z']
let upr = ['A' - 'Z']

rule token = parse ws+ { token lexbuf }
	| nl { Asmerror.incr_line_number (); token lexbuf }
	| "/*" { c_comment lexbuf; token lexbuf }
	| "(*" { o_comment lexbuf; token lexbuf }
	| "(" { LP }
	| ")" { RP }
	| "[" { LSB }
	| "]" { RSB }
	| "{" { LCB }
	| "}" { RCB }
	| "," { COMMA }
	| "::" { CONS }
	| ":" { COLON }
	| ";" { SEMICOLON }
	| "+" { PLUS }
	| "-" { MINUS }
	| "*" { STAR }
	| "=" { EQ }
	| "!=" { NEQ }
	| "<>" { NEQ }
	| "<" { LT }
	| "<u" { ULT }
	| ">" { GT }
	| ">u" { UGT }
	| "<=" { LE }
	| "<=u" { ULE }
	| ">=" { GE }
	| ">=u" { UGE }
	| "|" { PIPE }
	| "$" { DOLLAR }
	| "-->" { ARROW }
	| "type" { TYPE }
	| "block" { BLOCK }
	| "label" { LABEL }
	| "roll" { ROLL }
	| "unroll" { UNROLL }
	| "pack" { PACK }
	| "unpack" { UNPACK }
	| "split" { SPLIT }
	| "concat" { CONCAT }

	| "cast_to_tuple" { CAST_TO_TUPLE }
	| "cast_to_array" { CAST_TO_ARRAY }
	| "cast_to_stack" { CAST_TO_STACK }

	| "movl" { INST_MOVL }
	| "movb" { INST_MOVB }
	| "pushl" { INST_PUSHL }
	| "popl" { INST_POPL }
	| "call" { INST_CALL }
	| "jmp" { INST_JMP }
	| "jb" { INST_JB }
	| "jbe" { INST_JBE }
	| "ja" { INST_JA }
	| "jae" { INST_JAE }
	| "jz" { INST_JZ }
	| "jnz" { INST_JNZ }
	| "ret" { INST_RET }
	| "addl" { INST_ADDL }
	| "subl" { INST_SUBL }
	| "cmpl" { INST_CMPL }
	| "cmpb" { INST_CMPB }
	| "mull" { INST_MULL }
	| "imull" { INST_IMULL }

	| "byte" { TAG_BYTE }
	| "dword" { TAG_DWORD }

	| ".long" { AINST_LONG }
	| ".text" { AINST_TEXT }
	| ".data" { AINST_DATA }
	| ".globl" { AINST_GLOBL }

	| "%"(dig|lwr|upr)+ { let s = Lexing.lexeme lexbuf in let s = String.sub s 1 (String.length s - 1) in REG(s) }
	| (lwr|upr|'_') (dig|lwr|upr|'_')* { let s = Lexing.lexeme lexbuf in IDENT(s) }
	| "0x"(dig|lwr|upr)+ { let s = Lexing.lexeme lexbuf in UINT(Nativeint.of_string s) }
	| dig+ { let s = Lexing.lexeme lexbuf in UINT(Nativeint.of_string s) }
	| eof { EOF }
	| _ { failwith ("Uknown token : " ^ Lexing.lexeme lexbuf) }

and c_comment = parse "*/" { () }
	| "/*" { c_comment lexbuf; c_comment lexbuf }
	| "(*" { o_comment lexbuf; c_comment lexbuf }
	| nl { Asmerror.incr_line_number (); c_comment lexbuf }
	| eof { failwith "Comment not closed." }
	| _ { c_comment lexbuf }

and o_comment = parse "*)" { () }
	| "(*" { o_comment lexbuf; o_comment lexbuf }
	| "/*" { c_comment lexbuf; o_comment lexbuf }
	| nl { Asmerror.incr_line_number (); o_comment lexbuf }
	| eof { failwith "Comment not closed." }
	| _ { o_comment lexbuf }
