(*
 * Copyright 2005 Toshiyuki Maeda
 *
 * This file is part of TALK.
 *
 * TALK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * TALK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)

open Readable_types
open Readable_annotations

(*
type annot =
	| AnMem of integer * nativeint * integer * nativeint
*)
(*
type mem_annot = integer * nativeint * integer * nativeint
*)
type mem_annot = integer * nativeint

type op =
	| OpImd of string * bool
	| OpReg of string
	| OpMem of string * mem_annot

type raw_inst =
	| Inst0 of string
	| Inst1 of string * op
	| Inst2 of string * op * op
	| Inst3 of string * op * op * op

type named_type = {
	nt_name		: string;
	nt_definition	: named_type_definition;
}

type label_type =
	| LTData of heap_type
	| LTCode of state_type

type label = {
	l_name	: string;
	l_type	: label_type;
}

type debug_info = string

type inst =
	| IRaw of raw_inst * (debug_info list)
	| ILabel of string
	| IAnnot of annotation_inst

type block =
	| ALabelType of label
	| ANamedType of named_type
	| AProgram of string * inst list


type blocks = block list

let to_signed op = match op with
	| OpImd (n, _) ->
		OpImd (n, true)
	| _ ->
		op
