(*
 * Copyright 2005 Toshiyuki Maeda
 *
 * This file is part of TALK.
 *
 * TALK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * TALK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)

open Machine
open Instructions

open Disassemble_prefix
open Disassemble_tables
open Disassemble_common
open Disassemble_modrm
open Disassemble_nomodrm

(*************************************
	Disassemble opcode
 *************************************)

type modrm =
{
	modrm_mod	: int;
	modrm_rm	: int;
	modrm_reg	: int;
}

let get_2byte_op_0F_disas b =
	if is_2byte_modrm_op_0F b then
		get_2byte_modrm_op_0F b
	else
		get_2byte_no_modrm_op_0F b

let disas_2nd_opcode_0F_table =
	let sz = 256 in
	let a = Array.make sz (reserved_inst "simply reserved")in
	for b = 0 to (sz - 1) do
		a.(b) <- (get_2byte_op_0F_disas b)
	done;
	a
	
let get_2byte_opcode_0F dctx seg off =
	let b = get_byte seg off in
	let off = off + 1 in

	let disas = disas_2nd_opcode_0F_table.(b) in
	disas dctx seg off

let wrapper_for_2byte_op b =
	if b = 0x0f then
		get_2byte_opcode_0F
	else
		reserved_inst "2byte opcodes begin with '0x0f'"

let get_1byte_op_disas b =
	if is_1byte_modrm_op b then
		get_1byte_modrm_op b
	else
		get_1byte_no_modrm_op b

let get_disas b =
	if is_2byte_op_escape b then
		wrapper_for_2byte_op b
	else
		get_1byte_op_disas b

let disas_1st_opcode_table =
	let sz = 256 in
	let a = Array.make sz (reserved_inst "simply reserved")in
	for b = 0 to (sz - 1) do
		a.(b) <- (get_disas b)
	done;
	a

let get_opcode dctx seg off =
	let b = get_byte seg off in
	let off = off + 1 in

	let disas = disas_1st_opcode_table.(b) in
	disas dctx seg off



(*************************************
	Disassembler 
 *************************************)

let rec disassemble_sub op_attr ad_attr seg off inst_list =
	if (seg.seg_size = off) then inst_list

	else
	let start_off = off in

	let (off, prefixes) = get_prefixes seg off in

	let op_attr' = if has_op_override prefixes then (op_attr_override op_attr) else op_attr in
	let ad_attr' = if has_ad_override prefixes then (ad_attr_override ad_attr) else ad_attr in

	let dctx = { dctx_op_attr = op_attr'; dctx_ad_attr = ad_attr'; dctx_prefixes = prefixes; } in
	let (off, opcode) = get_opcode dctx seg off in

	let end_off = off in

	let inst = { inst_off = start_off; inst_size = (end_off - start_off); inst_opcode = opcode; } in

	let inst_list = inst :: inst_list in
	disassemble_sub op_attr ad_attr seg off inst_list

let disassemble op_attr ad_attr seg =
	List.rev (disassemble_sub op_attr ad_attr seg 0 [])
