(*
 * Copyright 2005 Toshiyuki Maeda
 *
 * This file is part of TALK.
 *
 * TALK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * TALK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)

open Read_elf_common

open Nativeint
open Machine
open Elf_header
open Elf_section
open Elf_segment

open Read_named_types
open Read_types
open Read_annots

(* Machine-dependent (for IA32) *)

let get_block_wrap_common block_getter seg program_headers va arg =
	let (a, sz) = arg in
	block_getter seg program_headers a va sz

let get_block_wrap = get_block_wrap_common get_block
let get_code_block_wrap = get_block_wrap_common get_code_block

let read_elf ba =

	let seg =
	{
		seg_array = ba;
		seg_offset = 0;
		seg_size = Bigarray.Array1.dim ba;
		seg_va = zero;
	} in

	let header = get_header seg in
	let section_headers = get_section_headers header seg in
	let program_headers = get_program_headers header seg in
	let strtab_seg = get_string_table header section_headers seg in

	let ectx =
	{
		ectx_header = header;
		ectx_section_headers = section_headers;
		ectx_program_headers = program_headers;
		ectx_strtab_seg = strtab_seg;
	} in

	let named_types = get_named_types ectx seg in
	let readable_heap_types = get_readable_heap_types ectx seg in
	let readable_state_types = get_readable_state_types ectx seg in
	let annots = get_annots ectx seg in

	let heap_types = Util.NativeintMap.map (fun (ht, i) ->
		let ht = Convert_from_readable.convert_heap_type [] ht in
		(ht, i)
	) readable_heap_types in

	let blks = Util.NativeintMap.mapi (get_block_wrap seg program_headers) heap_types in

	let sts = Util.NativeintMap.map (fun (st, i) ->
		let st = Convert_from_readable.convert_state_type [] st in
		(st, i)
	) readable_state_types in
	let c_blks = Util.NativeintMap.mapi (get_code_block_wrap seg program_headers) sts in

	named_types, blks, c_blks, annots

let read_elf_from_file filename =
	let ba = Util.read_file filename in
	read_elf ba

