(*
 * Copyright 2005 Toshiyuki Maeda
 *
 * This file is part of TALK.
 *
 * TALK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * TALK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)

(* Machine-dependent (for IA32) *)

open Nativeint
open Machine
open Elf_header
open Elf_section
open Elf_segment
open Typing_common

open Util

type elf_context =
{
	ectx_header		: elf_header;
	ectx_section_headers	: section_header list;
	ectx_program_headers	: program_header list;
	ectx_strtab_seg		: segment;
}

let is_section_by_name name strtab_seg section_header =
	let section_name = get_section_name section_header strtab_seg in
	section_name = name

let get_sections_by_name name empty get add ectx seg =
	let shs = List.filter (is_section_by_name name ectx.ectx_strtab_seg) ectx.ectx_section_headers in
	let sections = List.map (fun sh -> get_section_seg sh seg) shs in
	List.fold_right (fun section m ->
		let lst = get section in
		List.fold_right (fun elm m' ->
			let (key, value) = elm in
			add key value m') lst m
	) sections empty

let find_program_headers program_headers va sz =
	List.filter (in_program_header va sz) program_headers

let get_block_common block_init seg program_headers a va sz =
	let phs = find_program_headers program_headers va sz in
	match phs with
	| [ph] ->
		let off = to_int (sub va ph.p_vaddr) in
		let seg = get_segment_seg ph seg in
		let seg = get_subseg seg off sz in
		let seg = { seg with seg_va = va } in
		block_init seg a
	| [] -> failwithf "There are no segments!"
	| _ -> failwithf "Segments are overlapped!"

let get_block =
	get_block_common (fun seg ht ->
		{
			blk_segment = seg;
			blk_heap_type = ht;
		})

let get_code_block =
	get_block_common (fun seg st ->
		{
			cblk_segment = seg;
			cblk_code_type = Types.State st;
		})
